mod brutus;
mod chase;
mod diffuse;
mod human;
mod random;
pub mod util;

pub use brutus::Brutus;
pub use chase::Chase;
pub use diffuse::Diffuse;
pub use human::Human;
pub use random::Random;

use crate::{
	board::{Board, PlayerAction},
	util::Player,
};
use anyhow::Result;

pub trait Agent {
	fn name(&self) -> &'static str;
	fn player(&self) -> Player;
}

/// An agent that tries to minimize the value of a board.
pub trait MinimizerAgent: Agent {
	fn step_min(&mut self, board: &Board) -> Result<PlayerAction>;
}

/// An agent that tries to maximize the value of a board.
pub trait MaximizerAgent: Agent {
	fn step_max(&mut self, board: &Board) -> Result<PlayerAction>;
}
