use crate::{
	board::{Board, PlayerAction},
	util::{Player, Symb},
};
use anyhow::Result;
use rand::Rng;
use std::num::NonZeroU8;

use super::{Agent, MaximizerAgent, MinimizerAgent};

pub struct Random {
	player: Player,
}

impl Random {
	pub fn new(player: Player) -> Self {
		Self { player }
	}

	fn random_action(&self, board: &Board) -> PlayerAction {
		let mut rng = rand::thread_rng();
		let n = board.size();

		let pos = rng.gen_range(0..n);
		let symb = match rng.gen_range(0..4) {
			0 => {
				let n = rng.gen_range(0..=9);
				if n == 0 {
					Symb::Zero
				} else {
					Symb::Number(NonZeroU8::new(n).unwrap())
				}
			}
			1 => Symb::Div,
			2 => Symb::Minus,
			3 => Symb::Plus,
			4 => Symb::Times,
			_ => unreachable!(),
		};

		PlayerAction { symb, pos }
	}
}

impl Agent for Random {
	fn name(&self) -> &'static str {
		"Random"
	}

	fn player(&self) -> Player {
		self.player
	}
}

impl MinimizerAgent for Random {
	fn step_min(&mut self, board: &Board) -> Result<PlayerAction> {
		let mut action = self.random_action(board);
		while !board.can_play(&action) {
			action = self.random_action(board);
		}
		Ok(action)
	}
}

impl MaximizerAgent for Random {
	fn step_max(&mut self, board: &Board) -> Result<PlayerAction> {
		let mut action = self.random_action(board);
		while !board.can_play(&action) {
			action = self.random_action(board);
		}
		Ok(action)
	}
}
